/*
 *
 * VIEWFILE.PRG
 * ~~~~~~~~~~~~
 *
 * This program is a modeless replacement of FreshWins
 * modal wView() function.
 *
 * Robert DiFalco
 *
 */

#include "fresh.ch"        
#include "inkey.ch"
#include "wind.ch"

#define V_WIN     1
#define V_TEXT    2
#define V_TOTAL   3
#define V_TYPE    4
#define V_CURX    5
#define V_CURY    6
#define V_NEXTX   7
#define V_NEXTY   8
#define V_PAGEX   9
#define V_PAGEY   10
#define V_MAXY    11
#define V_SAVWR   12
#define V_SAVWB   13
#define V_SAVCLR  14
#define V_VSB     15
#define V_HSB     16
#define V_CLR     17
#define V_RECNO   18
#define V_FILE    19

#define MAX_VIEW  19

FUNCTION viewNew( cWin, uText, cClr, cDelim )

   local aView[ MAX_VIEW ]
   local n, nX, nY

   DEFAULT cClr TO wSetColor( cWin )

   aView[ V_WIN ] := wArray( cWin )

   aView[ V_TYPE  ] := valtype( uText )
   aView[ V_TOTAL ] := 0

   *-- Find out if a file was passed
   if aView[ V_TYPE ] == "C" .and. len( aView[ V_TYPE ] ) < 13
      aView[ V_TYPE ] := "F"

   endif

   /*
   * open the window if it is inactive.
   */
   if !isActive( aView[ V_WIN ] )
      wOpen( aView[ V_WIN ]:ID )

   endif

   do case
   case aView[ V_TYPE ] == "A"
      aView[ V_TEXT  ] := uText
      aView[ V_TOTAL ] := len( aView[ V_TEXT ] )

   case aView[ V_TYPE ] == "C"
      aView[ V_TOTAL ] := len( aView[ V_TEXT ] := Str2Arr( uText, cDelim ) )
      aView[ V_TYPE  ] := "A"

   case aView[ V_TYPE ] == "F"
      if f_use( alltrim( uText ) ) > 1

         aView[ V_FILE  ] := alltrim( uText ) ;;
         aView[ V_TOTAL ] := f_lastrec()      ;;

         wFooter( , "Reading line: " )        ;;
         nX := row( )                         ;;
         nY := col( )

         if aView[ V_TOTAL ] < 4096
            aView[ V_TEXT ] := array( aView[ V_TOTAL ] )         ;;
            f_goto( aView[ V_TOTAL ] )                           ;;
            for n := len( aView[ V_TEXT ] ) to 1 step -1         ;;
               wPutL( , 0, aView[ V_TEXT ][ n ] := f_readln( ) ) ;;
               wScroll( ,,,,,-1 )                                ;;
               PrintStr( nX, nY, str( n, 3 ) )                   ;;
               f_skip( -1 )                                      ;;

            next

            wFooter( , "" )         ;;
            f_use( )                ;;
            aView[ V_TYPE  ] := "A"

         else
            POPUP DIALOG NOBORDERS                        ;
                  TITLE  " Error! "                       ;
                  PROMPT  " Okay "                        ;
                  SAYTEXT "",                             ;
                          "Files in excess of 4096 lines",;
                          "are not yet supported",        ;
                          ""                              ;
                  TO      aView[ V_TOTAL ]
            aView[ V_TOTAL ] := 0

         endif
      else
         POPUP DIALOG NOBORDERS                                ;
               TITLE  " Error! "                               ;
               PROMPT  " Okay "                                ;
               SAYTEXT "",                                     ;
                       "Could not find file "+alltrim( uText ),;
                       ""                                      ;
               TO      aView[ V_TOTAL ]
         aView[ V_TOTAL ] := 0

      endif

   endcase

   if aView[ V_TOTAL ] > 0
      aView[ V_RECNO ]  := 1
      aView[ V_CURX ]   := 1 ; aView[ V_CURY ]   := 1
      aView[ V_NEXTX ]  := 0 ; aView[ V_NEXTY ]  := 0
      aView[ V_PAGEX ]  := 0 ; aView[ V_PAGEY ]  := 0
      aView[ V_MAXY  ]  := 172
      aView[ V_SAVWR ]  := aView[ V_WIN ]:box:r
      aView[ V_SAVWB ]  := aView[ V_WIN ]:box:b
      aView[ V_SAVCLR ] := setcolor( aView[ V_WIN ]:sayClr )
      aView[ V_CLR ]    := cClr
      aView[ V_VSB ]    := sbNew( SB_VSCROLL )
      aView[ V_HSB ]    := sbNew( SB_HSCROLL )

      *-- Create vertical scroll bar array
      sbClrSpec ( aView[ V_VSB ], aView[ V_CLR ] )

      *-- Create horizontal scroll bar array
      sbClrSpec ( aView[ V_HSB ], aView[ V_CLR ] )
      sbSetRange( aView[ V_HSB ], 1, aView[ V_MAXY ] )

      /*
      * Initially display one page of 
      * the file at screen pos aWin:txt:t, 
      * aWin:txt:l, 1 from left of first
      * line.
      */
      if !( aView[ V_WIN ]:condition == WZ_MIN )
         UpdateVars( aView )
         Show( aView, aView[ V_CURX ],;
               aView[ V_WIN ]:txt:t, aView[ V_WIN ]:txt:l,;
               aView[ V_PAGEX ], aView[ V_CURY ] )

      endif    
   endif

RETURN aView


FUNCTION viewUpdate( aMsg, aView )

   local nEvent  := SB_IDLE
   local n
      
   if ISNIL( aView )
      return NIL

   endif

   if !wOnTop( aView[ V_WIN ]:ID )
      w2Front( aView[ V_WIN ]:ID )

   endif

   if aView[ V_TOTAL ] == 0
      return FALSE

   endif

   *-- get the scroll bar event for both vert and horz
   setcolor( aView[ V_CLR ] )

   aMsg := vTranslateEvent( aMsg, aView )
 
   /*
   * Make sure we are working with
   * the correct reference. If a window
   * is maximized, we will loose that
   * reference.
   */
   aView[ V_WIN ] := wArray( aView[ V_WIN ]:ID )

   do case
   case !( aMsg[ E_WIN ] == aView[ V_WIN ]:ID ) 
   case !isActive( aView[ V_WIN ] )
   case aView[ V_WIN ]:condition == WZ_MIN
      aView[ V_SAVWB ] := -1

   case !( aView[ V_SAVWB ] == aView[ V_WIN ]:box:b ) .or. ;
        !( aView[ V_SAVWR ] == aView[ V_WIN ]:box:r )
      /*
      * Reset motion window
      * motion indicators.
      */
      aView[ V_SAVWB ] := aView[ V_WIN ]:box:b
      aView[ V_SAVWR ] := aView[ V_WIN ]:box:r

      /* 
      * Update page sizes, display scroll 
      * bars, reset scroll bar positions and
      * and redisplay the file text.
      */
      UpDateVars( aView )
      Show( aView, aView[ V_CURX ], ;
            aView[ V_WIN ]:txt:t, aView[ V_WIN ]:txt:l,;
            aView[ V_PAGEX ], aView[ V_CURY ] )

   otherwise
      if aMsg[ E_MSG ] == FWM_KEYBOARD
        nEvent := sbGetEvent( aView[ V_VSB ], aView[ V_HSB ],;
                              aMsg[ E_P1 ], 0 )

      elseif aMsg[ E_MSG ] == FWM_MOUSE
        nEvent := sbGetEvent( aView[ V_VSB ], aView[ V_HSB ],;
                              0, aMsg[ E_P1 ] )

      else
        nEvent := sbGetEvent( aView[ V_VSB ], aView[ V_HSB ], 0, 0 )

      endif
   endcase

   do case
   case nEvent == SB_THUMBTRACK  // user is dragging Hor|vert thumb
      aView[ V_NEXTX ] := int( sbWhere( aView[ V_VSB ] ) )
      aView[ V_NEXTY ] := int( sbWhere( aView[ V_HSB ] ) )

      if !( aView[ V_NEXTX ] == aView[ V_CURX ] )
         aView[ V_CURX ] := aView[ V_NEXTX ]
         Show( aView, aView[ V_CURX ],;
               aView[ V_WIN ]:txt:t, aView[ V_WIN ]:txt:l,;
               aView[ V_PAGEX ], aView[ V_CURY ] )

      endif

      if !( aView[ V_NEXTY ] == aView[ V_CURY ] )
         aView[ V_CURY ] := aView[ V_NEXTY ]
         Show( aView, aView[ V_CURX ],;
               aView[ V_WIN ]:txt:t, aView[ V_WIN ]:txt:l,;
               aView[ V_PAGEX ], aView[ V_CURY ] )

      endif              

   case nEvent == SB_HOME
      aView[ V_CURY ] := 1
      Show( aView, aView[ V_CURX ],;
            aView[ V_WIN ]:txt:t, aView[ V_WIN ]:txt:l,;
            aView[ V_PAGEX ], aView[ V_CURY ] )
      sbSetPos( aView[ V_HSB ], aView[ V_CURY ] )

   case nEvent == SB_TOP
      aView[ V_CURX ] := aView[ V_CURY ] := 1
      Show( aView, aView[ V_CURX ],;
            aView[ V_WIN ]:txt:t, aView[ V_WIN ]:txt:l,;
            aView[ V_PAGEX ], aView[ V_CURY ] )   
      sbSetPos( aView[ V_VSB ], aView[ V_CURX ] )
      sbSetPos( aView[ V_HSB ], aView[ V_CURY ] )

   case nEvent == SB_BOTTOM
      aView[ V_CURX ] := aView[ V_TOTAL ] - ;
                        ( aView[ V_WIN ]:txt:r - ;
                          aView[ V_WIN ]:txt:l + 1 )
      aView[ V_CURY ] := 1
      Show( aView, aView[ V_CURX ],;
            aView[ V_WIN ]:txt:t, aView[ V_WIN ]:txt:l,;
            aView[ V_PAGEX ], aView[ V_CURY ] )
      sbSetPos( aView[ V_VSB ], aView[ V_CURX ] )
      sbSetPos( aView[ V_HSB ], aView[ V_CURY ] )

   case nEvent == SB_LINEUP  // K_UP or mouse on up arrow
      if !( aView[ V_CURX ] < 2 )
         MsHideAt( aView[ V_WIN ]:txt:t, aView[ V_WIN ]:txt:l,;
                   aView[ V_WIN ]:txt:b, aView[ V_WIN ]:txt:r )
         scroll  ( aView[ V_WIN ]:txt:t, aView[ V_WIN ]:txt:l,;
                   aView[ V_WIN ]:txt:b, aView[ V_WIN ]:txt:r, -1 )

         MsShow( )
         Show( aView, --aView[ V_CURX ],;
               aView[ V_WIN ]:txt:t, aView[ V_WIN ]:txt:l,;
               1, aView[ V_CURY ] )
         sbSetPos( aView[ V_VSB ], aView[ V_CURX ] )

      endif

   case nEvent == SB_LINEDOWN  // K_DOWN or mouse on down arrow
      if aView[ V_CURX ] + ;
         ( aView[ V_WIN ]:txt:b - aView[ V_WIN ]:txt:t ) - 1;
         <= aView[ V_TOTAL ]
         MsHideAt( aView[ V_WIN ]:txt:t, aView[ V_WIN ]:txt:l,;
                   aView[ V_WIN ]:txt:b, aView[ V_WIN ]:txt:r )
         scroll  ( aView[ V_WIN ]:txt:t, aView[ V_WIN ]:txt:l,;
                   aView[ V_WIN ]:txt:b, aView[ V_WIN ]:txt:r, 1 )
         Show( aView, aView[ V_CURX ] + aView[ V_PAGEX ],;
               aView[ V_WIN ]:txt:b, aView[ V_WIN ]:txt:l,;
               1, aView[ V_CURY ] )
         MsShow( )
         sbSetPos( aView[ V_VSB ], ++aView[ V_CURX ] )

      endif

   case nEvent == SB_LEFT    // K_LEFT or mouse on left arrow
      if aView[ V_CURY ] > 1
         Show( aView, aView[ V_CURX ],;
               aView[ V_WIN ]:txt:t, aView[ V_WIN ]:txt:l,;
               aView[ V_PAGEX ], --aView[ V_CURY ] )
         sbSetPos( aView[ V_HSB ], aView[ V_CURY ] )

      endif

   case nEvent == SB_RIGHT  // K_RIGHT or mouse on right arrow
      if aView[ V_CURY ] < aView[ V_MAXY ]
         Show( aView, aView[ V_CURX ],;
               aView[ V_WIN ]:txt:t, aView[ V_WIN ]:txt:l,;
               aView[ V_PAGEX ], ++aView[ V_CURY ] )
         sbSetPos( aView[ V_HSB ], aView[ V_CURY ] )

      endif

   case nEvent == SB_PGUP  // K_PGUP or mouse click on bar above thumb
      aView[ V_CURX ] := iif( aView[ V_CURX ] > aView[ V_PAGEX ], aView[ V_CURX ] - aView[ V_PAGEX ], 1 )
      Show( aView, aView[ V_CURX ],;
            aView[ V_WIN ]:txt:t, aView[ V_WIN ]:txt:l,;
            aView[ V_PAGEX ], aView[ V_CURY ] )
      sbSetPos( aView[ V_VSB ], aView[ V_CURX ] )

   case nEvent == SB_PGDN  // K_PGDN or mouse click on bar below thumb
      aView[ V_CURX ] := iif( aView[ V_CURX ] + aView[ V_PAGEX ] <= aView[ V_TOTAL ],;
                    aView[ V_CURX ] + aView[ V_PAGEX ],           ;
                    aView[ V_TOTAL ] - aView[ V_PAGEX ]  )
      Show( aView, aView[ V_CURX ],;
            aView[ V_WIN ]:txt:t, aView[ V_WIN ]:txt:l,;
            aView[ V_PAGEX ], aView[ V_CURY ] )
      sbSetPos( aView[ V_VSB ], aView[ V_CURX ] )

   case nEvent == SB_PANLEFT // mouse on bar left of thumb
      aView[ V_CURY ] := iif( aView[ V_CURY ] > aView[ V_PAGEY ], aView[ V_CURY ] - aView[ V_PAGEY ], 1 )
      Show( aView, aView[ V_CURX ],;
            aView[ V_WIN ]:txt:t, aView[ V_WIN ]:txt:l,;
            aView[ V_PAGEX ], aView[ V_CURY ] )
      sbSetPos( aView[ V_HSB ], aView[ V_CURY ] )

   case nEvent == SB_PANRIGHT   // mouse on bar right of thumb
      aView[ V_CURY ] := iif( aView[ V_CURY ] < aView[ V_MAXY ], aView[ V_CURY ] + aView[ V_PAGEY ], aView[ V_MAXY ] )
      Show( aView, aView[ V_CURX ],;
            aView[ V_WIN ]:txt:t, aView[ V_WIN ]:txt:l,;
            aView[ V_PAGEX ], aView[ V_CURY ] )
      sbSetPos( aView[ V_HSB ], aView[ V_CURY ] )

   endcase
         
   setcolor( aView[ V_SAVCLR ] )

RETURN TRUE

STATIC FUNCTION Show( aView, nRec, nX, nY, nLen, nOff )

   local n

   MsHide( nX, nY, nX, + nLen, aView[ V_WIN ]:txt:r )

   for n := 1 TO nLen
      setpos( nX++, nY )
      if nRec <= aView[ V_TOTAL ]
         dispout( padr( subs( aView[ V_TEXT ][ nRec ], aView[ V_CURY ] ),;
                        aView[ V_WIN ]:txt:r - aView[ V_WIN ]:txt:l + 1 ) )
         ++nRec

      else
         dispout( space( aView[ V_WIN ]:txt:r - aView[ V_WIN ]:txt:l + 1 ) )

      endif
   next   

   MsShow( )

RETURN NIL


STATIC FUNCTION UpdateVars( aView )

   aView[ V_PAGEX ] := ( aView[ V_WIN ]:txt:b - aView[ V_WIN ]:txt:t + 1 )
   aView[ V_PAGEY ] := ( aView[ V_WIN ]:txt:r - aView[ V_WIN ]:txt:l + 1 )

   sbSetRange( aView[ V_VSB ],;
               1,             ;
               aView[ V_TOTAL ] - ( ( aView[ V_WIN ]:txt:b - aView[ V_WIN ]:txt:t ) - 3 ) )

   sbDisplay( aView[ V_VSB ], aView[ V_WIN ]:txt:t,;
                              aView[ V_WIN ]:box:r,;
                              aView[ V_WIN ]:txt:b - aView[ V_WIN ]:txt:t )
   sbDisplay( aView[ V_HSB ], aView[ V_WIN ]:box:b,;
                              aView[ V_WIN ]:txt:l,;
                              aView[ V_WIN ]:txt:r - aView[ V_WIN ]:txt:l )
   sbSetPos ( aView[ V_VSB ], aView[ V_CURX ] )
   sbSetPos ( aView[ V_HSB ], aView[ V_CURY ] )

RETURN NIL


STATIC FUNCTION vTranslateEvent( aMsg, aView )

   if aMsg[ E_WIN ] == aView[ V_WIN ]:ID
      do case
      case aMsg[ E_MSG ] == FWM_MOVE
         wMove( )

      case aMsg[ E_MSG ] == FWM_SIZE
         wReSize( )

      case aMsg[ E_MSG ] == FWM_MAX
         wZoom( , WZ_MAX )

      case aMsg[ E_MSG ] == FWM_MIN
         wZoom( , WZ_MIN )

      case aMsg[ E_MSG ] == FWM_CLOSE
         wClose( aMsg[ E_WIN ] )

      endcase
   endif

RETURN aMsg

         

